package java.beans;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.util.Vector;

public class PropertyEditorSupport implements PropertyEditor
{
    private Object value;
    private Object source;
    private Vector<PropertyChangeListener> listeners;
    
    public PropertyEditorSupport() {
        this.setSource(this);
    }
    
    public PropertyEditorSupport(Object source) {
        if (source == null) {
            throw new NullPointerException();
        }
        this.setSource(source);
        return;
    }
    
    public Object getSource() {
        return this.source;
    }
    
    public void setSource(Object source) {
        this.source = source;
    }
    
    @Override
    public void setValue(Object value) {
        this.value = value;
        this.firePropertyChange();
    }
    
    @Override
    public Object getValue() {
        return this.value;
    }
    
    @Override
    public boolean isPaintable() {
        return false;
    }
    
    @Override
    public void paintValue(Graphics graphics, Rectangle rectangle) {
    }
    
    @Override
    public String getJavaInitializationString() {
        return "???";
    }
    
    @Override
    public String getAsText() {
        return (this.value != null) ? this.value.toString() : null;
    }
    
    @Override
    public void setAsText(String s) throws IllegalArgumentException {
        if ((this.value instanceof String)) {
            this.setValue(s);
            return;
        }
        throw new IllegalArgumentException(s);
    }
    
    @Override
    public String[] getTags() {
        return null;
    }
    
    @Override
    public Component getCustomEditor() {
        return null;
    }
    
    @Override
    public boolean supportsCustomEditor() {
        return false;
    }
    
    @Override
    public synchronized void addPropertyChangeListener(PropertyChangeListener obj) {
        if (this.listeners == null) {
            this.listeners = new Vector();
        }
        this.listeners.addElement(obj);
    }
    
    @Override
    public synchronized void removePropertyChangeListener(PropertyChangeListener obj) {
        if (this.listeners == null) {
            return;
        }
        this.listeners.removeElement(obj);
        return;
    }
    
    public void firePropertyChange() {
        Vector<PropertyChangeListener> unsafeClone;
        synchronized (this) {
            if (this.listeners == null) {
                return;
            }
            unsafeClone = this.unsafeClone(this.listeners);
        }
        PropertyChangeEvent propertyChangeEvent = new PropertyChangeEvent(this.source, null, null, null);
        for (int i = 0; i < unsafeClone.size(); ++i) {
            ((PropertyChangeListener)unsafeClone.elementAt(i)).propertyChange(propertyChangeEvent);
        }
    }
    
    private <T> Vector<T> unsafeClone(Vector<T> vector) {
        return (Vector)vector.clone();
    }
}
